package net.knarcraft.permissionsigns.listener;

import net.knarcraft.permissionsigns.PermissionSigns;
import net.knarcraft.permissionsigns.container.PermissionSign;
import net.knarcraft.permissionsigns.container.SignCreationRequest;
import net.knarcraft.permissionsigns.formatting.TranslatableMessage;
import net.knarcraft.permissionsigns.formatting.Translator;
import net.knarcraft.permissionsigns.manager.EconomyManager;
import net.knarcraft.permissionsigns.manager.PermissionManager;
import net.knarcraft.permissionsigns.manager.SignManager;
import org.bukkit.Material;
import org.bukkit.Tag;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;

import java.util.Arrays;

/**
 * A listener for all events related to signs
 */
public class SignListener implements Listener {

    /**
     * This event handler detects if a player clicks a button or a sign
     *
     * @param event <p>The player interact event which was triggered</p>
     */
    @EventHandler
    public void onPlayerInteract(PlayerInteractEvent event) {
        Player player = event.getPlayer();
        Block block = event.getClickedBlock();

        if (block == null) {
            return;
        }

        Material material = block.getBlockData().getMaterial();
        if (!Tag.SIGNS.isTagged(material) && !Tag.WALL_SIGNS.isTagged(material)) {
            return;
        }

        Sign sign = (Sign) block.getState();
        if (event.getAction() == Action.RIGHT_CLICK_BLOCK) {
            handleSignRightClick(sign, player);
        } else if (event.getAction() == Action.LEFT_CLICK_BLOCK) {
            handleSignLeftClick(sign, player, event);
        }
    }

    /**
     * Handles the left click of a sign that might be a permission sign
     *
     * @param sign   <p>The clicked sign</p>
     * @param player <p>The player that clicked the sign</p>
     * @param event  <p>The triggered interaction event</p>
     */
    private void handleSignLeftClick(Sign sign, Player player, PlayerInteractEvent event) {
        boolean registered = SignManager.getSign(sign.getLocation()) != null;
        if (!registered) {
            return;
        }
        if (!player.hasPermission("permissionsigns.admin")) {
            event.setCancelled(true);
            player.sendMessage(Translator.getTranslatedMessage(TranslatableMessage.PERMISSION_SIGN_DESTROY_DENY));
        } else {
            SignManager.removeSign(sign.getLocation());
            player.sendMessage(Translator.getTranslatedMessage(TranslatableMessage.PERMISSION_SIGN_REMOVED));
        }
    }

    /**
     * Handles the right-clicking action
     *
     * @param sign   <p>The clicked sign</p>
     * @param player <p>The player that clicked the sign</p>
     */
    private void handleSignRightClick(Sign sign, Player player) {
        //Check if the sign is a registered permission sign
        PermissionSign permissionSign = SignManager.getSign(sign.getLocation());
        if (permissionSign != null) {
            handlePermissionSignInteract(permissionSign, player);
            return;
        }

        //Check if the player has a creation request that can be fulfilled
        SignCreationRequest request = PermissionSigns.getSignCreationRequest(player.getUniqueId());
        if (request != null) {
            registerPermissionSign(sign, request, player);
        }
    }

    /**
     * Handles the interaction with a permission sign
     *
     * @param permissionSign <p>The permission sign that was interacted with</p>
     * @param player         <p>The player that interacted with the permission sign</p>
     */
    private void handlePermissionSignInteract(PermissionSign permissionSign, Player player) {
        //TODO: Check if the player has the permissions for sale. If it has, just explain that it does
        if (EconomyManager.canAfford(player, permissionSign.getCost())) {
            EconomyManager.withdraw(player, permissionSign.getCost());
            for (String permissionString : permissionSign.getPermissionNodes()) {
                PermissionManager.addPermission(player, permissionString, permissionSign.getDuration() == 0);
            }
            //TODO: Tell the player that they've been granted the new permissions, possibly including the full nodes?
        } else {
            //TODO: Tell the player that they cannot afford the transaction
        }
    }

    /**
     * Registers a new permission sign if the given sign is empty
     *
     * @param sign    <p>The sign to register as a permission sign</p>
     * @param request <p>The creation request to fulfill</p>
     * @param player  <p>The player that interacted with the sign</p>
     */
    private void registerPermissionSign(Sign sign, SignCreationRequest request, Player player) {
        String[] lines = sign.getLines();
        //Don't allow non-empty signs to be overwritten
        if (!Arrays.stream(lines).allMatch(String::isEmpty)) {
            return;
        }

        //Register the sign and remove the request
        PermissionSign permissionSign = request.getPermissionSign();
        permissionSign.setSignLocation(sign.getLocation());
        SignManager.addSign(permissionSign);
        PermissionSigns.cancelSignCreationRequest(player.getUniqueId());

        //Display the permission sign information on the sign
        String[] signLines = permissionSign.getSignLines();
        for (int i = 0; i < signLines.length; i++) {
            sign.setLine(i, signLines[i]);
        }
        sign.update();
        //TODO: Display text in the chat explaining that the permission sign has been created
    }

}
